"""Host Information."""

import logging
import threading
import time
from os import walk
from pathlib import Path

import serial  # type: ignore

from camera_multi_tester.modules.networkhandler import NetworkMonitor


class CMTMonitor:
    """CMTMonitor (Camera Multi Tester)."""

    def __init__(self) -> None:
        """The CMTMonitor constructor."""
        self.cmth = CMTHandler()
        self.uart: str = ""  # Example: '/dev/ttyUSB0'
        self.netwmon: NetworkMonitor
        t = threading.Thread(target=self.cmtd, args=([]))
        t.start()
        return

    def set_netwmon(self, netwmon: NetworkMonitor) -> None:
        """Set the NetworkMonitor object to get network devices."""
        self.netwmon = netwmon
        return

    def cmtd(self) -> None:
        """CMT daemon to monitor UART status and CMT commands."""
        uart: str = ""
        while True:
            time.sleep(0.3)
            # CMT (uart) detection
            uart = self._get_uart()
            if self.uart != uart and not uart:
                logging.warning("Camera Multi Tester (UART) not found")
                self.cmth.stop_cmt()
            if self.uart != uart and uart:
                logging.info("Camera Multi Tester (UART) detected!")
                self.cmth.start_cmt(uart)
            self.uart = uart
            # Network information
            ip = "Not found"
            if self.netwmon:
                netwdevs = self.netwmon.get_netwdevs()
                if "eth0" in netwdevs.keys():
                    ip = netwdevs["eth0"]["ip"]
            # CMT communication
            if uart:
                self.cmth.cmd_cmt(ip)

    def _get_uart_dict(self) -> dict[str, str]:
        dev: dict[str, str] = {}
        # [] if no file
        ser: list[str] = next(walk("/dev/serial/by-id"), (None, None, []))[
            2
        ]  # type: ignore
        for d in ser:
            if "FTDI_TTL232R-3V3" in d:
                p = "/dev/serial/by-id/" + d
                serial = Path(p).resolve()
                dev.update({"0": str(serial)})
        return dev

    def _get_uart(self) -> str:
        dev: str = ""
        # [] if no file
        ser: list[str] = next(walk("/dev/serial/by-id"), (None, None, []))[
            2
        ]  # type: ignore
        for d in ser:
            if "FTDI_TTL232R-3V3" in d:
                p = "/dev/serial/by-id/" + d
                serial = Path(p).resolve()
                dev = str(serial)
        return dev


class CMTHandler:
    """CMTHandler (Camera Multi Tester)."""

    def __init__(self) -> None:
        """The CMTHandler constructor."""
        self.ser: serial.Serial
        self.line: bytes
        return

    def start_cmt(self, tty: str) -> None:
        """CMT UART device open."""
        logging.debug(f"Opening serial port '{tty}'...")
        self.ser = serial.Serial(tty, 9600, timeout=0.1)
        return

    def stop_cmt(self) -> None:
        """CMT UART device close."""
        logging.debug(f"Closing serial port '{self.ser.name}'...")
        self.ser.close()
        return

    def cmd_cmt(self, ip: str) -> None:
        """CMT command receiver and parser."""
        line = self.ser.readline()
        if line:
            logging.debug(f"CMT (cmd):\n{line}")
            # 'iprequest' command handler
            if line.decode("utf-8") == "iprequest\r\n":
                msg = bytes(ip + "\r\n", "utf-8")
                self.ser.write(msg)
                logging.debug(f"Sending:\n{msg!r}")
